@@ -1,11 +1,11 @@
-# $Id: Filter.pm 2357 2008-06-20 17:41:54Z rcaputo $
+# $Id: Filter.pm 2490 2009-03-02 20:52:40Z rcaputo $
 
 package POE::Filter;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2357 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2490 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Carp qw(croak);
 
@@ -166,14 +166,16 @@ The L</SYNOPSIS> shows get_one_start() in use.
 
 get_one() parses zero or one complete item from the filter's internal
 buffer.  The data is returned as an ARRAYREF suitable for passing to
-another filter or a POE::Wheel object.
+another filter or a POE::Wheel object.  Filters will return empty
+ARRAYREFs if they don't have enough raw data to build a complete item.
 
 get_one() is the lazy form of get().  It only parses only one item at
 a time from the filter's buffer.  This is vital for applications that
 may switch filters in mid-stream, as it ensures that the right filter
 is in use at any given time.
 
-The L</SYNOPSIS> shows get_one() in use.
+The L</SYNOPSIS> shows get_one() in use.  Note how it assumes the
+return is always an ARRAYREF, and it implicitly handles empty ones.
 
 =head2 get ARRAYREF
 
@@ -183,6 +185,9 @@ filter's internal buffer.  It then parses as many full items as
 possible from the buffer and returns them in another array reference.
 Any unprocessed data remains in the filter's buffer for the next call.
 
+As with get_one(), get() will return an empty array reference if the
+filter doesn't contain enough raw data to build a complete item.
+
 In fact, get() is implemented in POE::Filter in terms of
 get_one_start() and get_one().
 
@@ -359,3 +364,4 @@ Please see L<POE> for more information about authors and contributors.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
